@@ -100,7 +100,7 @@ int my_onReceive(swServer *serv, swEventData *req)
     printf("onReceive[%d]: ip=%s|port=%d Data=%s|Len=%d\n", g_receive_count, swConnection_get_ip(conn),
             swConnection_get_port(conn), req->data, req->info.len);
 
-    int n = snprintf(resp_data, SW_IPC_BUFFER_SIZE, "Server: %*s\n", req->info.len, req->data);
+    int n = snprintf(resp_data, SW_IPC_BUFFER_SIZE, "Server: %.*s\n", req->info.len, req->data);
     ret = serv->send(serv, req->info.fd, resp_data, n);
     if (ret < 0)
     {
@@ -149,10 +149,10 @@ int my_onPacket(swServer *serv, swEventData *req)
         length = packet->length;
     }
 
-    printf("Packet[client=%s:%d, %d bytes]: data=%*s\n", address, port, length, length, data);
+    printf("Packet[client=%s:%d, %d bytes]: data=%.*s\n", address, port, length, length, data);
 
     char resp_data[SW_IPC_BUFFER_SIZE];
-    int n = snprintf(resp_data, SW_IPC_BUFFER_SIZE, "Server: %*s", length, data);
+    int n = snprintf(resp_data, SW_IPC_BUFFER_SIZE, "Server: %.*s", length, data);
 
     //udp ipv4
     if (req->info.type == SW_EVENT_UDP)
@@ -184,12 +184,12 @@ int my_onPacket(swServer *serv, swEventData *req)
 
 void my_onStart(swServer *serv)
 {
-    sw_log("Server is running");
+    swNotice("Server is running");
 }
 
 void my_onShutdown(swServer *serv)
 {
-    sw_log("Server is shutdown\n");
+    swNotice("Server is shutdown");
 }
 
 void my_onConnect(swServer *serv, swDataHead *info)
