@@ -9,6 +9,9 @@ $table->set('tianfenghan@qq.com', array('id' => 145, 'name' => 'rango1', 'num' =
 $table->set('350749960@qq.com', array('id' => 358, 'name' => "Rango2", 'num' => 3.1415));
 $table->set('hello@qq.com', array('id' => 189, 'name' => 'rango3', 'num' => 3.1415));
 
+var_dump($table->get('350749960@qq.com'));
+var_dump($table->get('350749960@qq.com', 'name'));
+
 foreach($table as $key => $value)
 {
     var_dump($key, $value);
@@ -21,7 +24,6 @@ foreach($table as $key => $value)
     var_dump($key, $value);
 }
 echo "======================= Total Elements: {$table->count()} ============================\n";
-
 $ret = $table->del('a invalid key'); // delete a invalid element
 var_dump($ret);
 foreach($table as $key => $value)
