@@ -45,7 +45,6 @@ enum swHttp_proxy_state
 struct _http_proxy
 {
     uint8_t state;
-    uint8_t ssl;
     int proxy_port;
     char *proxy_host;
     char *user;
@@ -188,7 +187,6 @@ int swDNSResolver_free();
 typedef struct _swStream
 {
     swString *buffer;
-    uint32_t session_id;
     uint8_t cancel;
     void (*response)(struct _swStream *stream, char *data, uint32_t length);
     swClient client;
