@@ -182,7 +182,7 @@ int swString_extend(swString *str, size_t new_size)
 
 char* swString_alloc(swString *str, size_t __size)
 {
-    if (str->length + __size < str->size)
+    if (str->length + __size > str->size)
     {
         if (swString_extend_align(str, str->length + __size) < 0)
         {
@@ -195,7 +195,7 @@ char* swString_alloc(swString *str, size_t __size)
     return tmp;
 }
 
-uint32_t swoole_utf8_decode(u_char **p, size_t n)
+uint32_t swoole_utf8_decode(uchar **p, size_t n)
 {
     size_t len;
     uint32_t u, i, valid;
@@ -252,9 +252,9 @@ uint32_t swoole_utf8_decode(u_char **p, size_t n)
     return 0xffffffff;
 }
 
-size_t swoole_utf8_length(u_char *p, size_t n)
+size_t swoole_utf8_length(uchar *p, size_t n)
 {
-    u_char c, *last;
+    uchar c, *last;
     size_t len;
 
     last = p + n;
