@@ -26,6 +26,9 @@
 
 using namespace swoole;
 
+#define MAGIC_STRING  "swoole_coroutine#5652a7fb2b38be"
+#define START_OFFSET  (64 * 1024)
+
 Context::Context(size_t stack_size, coroutine_func_t fn, void* private_data) :
         fn_(fn), stack_size_(stack_size), private_data_(private_data)
 {
@@ -42,6 +45,15 @@ Context::Context(size_t stack_size, coroutine_func_t fn, void* private_data) :
 #endif
     ctx_ = make_fcontext(sp, stack_size_, (void (*)(intptr_t))&context_func);
 
+#ifdef SW_LOG_TRACE_OPEN
+    size_t offset = START_OFFSET;
+    while (offset <= stack_size)
+    {
+        memcpy((char*) sp - offset + (sizeof(MAGIC_STRING) -1), SW_STRL(MAGIC_STRING));
+        offset *= 2;
+    }
+#endif
+
     uint32_t protect_page = get_protect_stack_page();
     if (protect_page)
     {
@@ -69,6 +81,27 @@ Context::~Context()
     }
 }
 
+#ifdef SW_LOG_TRACE_OPEN
+ssize_t Context::get_stack_usage()
+{
+    size_t offset = START_OFFSET;
+    size_t retval = START_OFFSET;
+
+    void* sp = (void*) ((char*) stack_ + stack_size_);
+
+    while (offset < stack_size_)
+    {
+        if (memcmp((char*) sp - offset + (sizeof(MAGIC_STRING) - 1), SW_STRL(MAGIC_STRING)) != 0)
+        {
+            retval = offset * 2;
+        }
+        offset *= 2;
+    }
+
+    return retval;
+}
+#endif
+
 bool Context::SwapIn()
 {
     jump_fcontext(&swap_ctx_, ctx_, (intptr_t) this, true);
