@@ -294,6 +294,12 @@ static int swReactorKqueue_wait(swReactor *reactor, struct timeval *timeo)
             t.tv_nsec = (reactor->timeout_msec - t.tv_sec * 1000) * 1000 * 1000;
             t_ptr = &t;
         }
+        else if (reactor->defer_tasks)
+        {
+            t.tv_sec = 0;
+            t.tv_nsec = 0;
+            t_ptr = &t;
+        }
         else
         {
             t_ptr = NULL;
@@ -318,7 +324,7 @@ static int swReactorKqueue_wait(swReactor *reactor, struct timeval *timeo)
             {
                 reactor->onTimeout(reactor);
             }
-            continue;
+            SW_REACTOR_CONTINUE;
         }
 
         for (i = 0; i < n; i++)
@@ -382,6 +388,10 @@ static int swReactorKqueue_wait(swReactor *reactor, struct timeval *timeo)
                 {
                     swWarn("unknown event filter[%d].", object->events[i].filter);
                 }
+                if (!event.socket->removed && (event.socket->events & SW_EVENT_ONCE))
+                {
+                    reactor->del(reactor, event.fd);
+                }
             }
         }
 
@@ -389,10 +399,7 @@ static int swReactorKqueue_wait(swReactor *reactor, struct timeval *timeo)
         {
             reactor->onFinish(reactor);
         }
-        if (reactor->once)
-        {
-            break;
-        }
+        SW_REACTOR_CONTINUE;
     }
     return 0;
 }
