@@ -270,7 +270,6 @@ static int swServer_start_check(swServer *serv)
             ls->protocol.package_max_length = SW_BUFFER_MIN_SIZE;
         }
     }
-
 #ifdef SW_USE_OPENSSL
     /**
      * OpenSSL thread-safe
@@ -1395,7 +1394,7 @@ SW_API int swServer_add_hook(swServer *serv, enum swServer_hook_type type, swCal
 /**
  * Return the number of ports successfully
  */
-int swserver_add_systemd_socket(swServer *serv)
+int swServer_add_systemd_socket(swServer *serv)
 {
     char *e = getenv("LISTEN_PID");
     if (!e)
@@ -1520,8 +1519,7 @@ int swserver_add_systemd_socket(swServer *serv)
         //dgram socket, setting socket buffer size
         if (swSocket_is_dgram(ls->type))
         {
-            setsockopt(sock, SOL_SOCKET, SO_SNDBUF, &ls->socket_buffer_size, sizeof(int));
-            setsockopt(sock, SOL_SOCKET, SO_RCVBUF, &ls->socket_buffer_size, sizeof(int));
+            swSocket_set_buffer_size(sock, ls->socket_buffer_size);
         }
         //O_NONBLOCK & O_CLOEXEC
         swoole_fcntl_set_option(sock, 1, 1);
@@ -1617,8 +1615,7 @@ swListenPort* swServer_add_port(swServer *serv, int type, const char *host, int
     //dgram socket, setting socket buffer size
     if (swSocket_is_dgram(ls->type))
     {
-        setsockopt(sock, SOL_SOCKET, SO_SNDBUF, &ls->socket_buffer_size, sizeof(int));
-        setsockopt(sock, SOL_SOCKET, SO_RCVBUF, &ls->socket_buffer_size, sizeof(int));
+        swSocket_set_buffer_size(sock, ls->socket_buffer_size);
     }
     //O_NONBLOCK & O_CLOEXEC
     swoole_fcntl_set_option(sock, 1, 1);
@@ -1720,7 +1717,7 @@ static void swServer_signal_handler(int sig)
         }
         else
         {
-            kill(serv->gs->manager_pid, sig);
+            swKill(serv->gs->manager_pid, sig);
         }
         break;
     default:
@@ -1732,11 +1729,11 @@ static void swServer_signal_handler(int sig)
             for (i = 0; i < SwooleG.serv->worker_num + serv->task_worker_num + SwooleG.serv->user_worker_num; i++)
             {
                 worker = swServer_get_worker(SwooleG.serv, i);
-                kill(worker->pid, SIGRTMIN);
+                swKill(worker->pid, SIGRTMIN);
             }
             if (SwooleG.serv->factory_mode == SW_MODE_PROCESS)
             {
-                kill(serv->gs->manager_pid, SIGRTMIN);
+                swKill(serv->gs->manager_pid, SIGRTMIN);
             }
             swServer_reopen_log_file(SwooleG.serv);
         }
@@ -1768,7 +1765,7 @@ static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, i
     if (ls->open_tcp_nodelay && ls->type != SW_SOCK_UNIX_STREAM)
     {
         int sockopt = 1;
-        if (setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &sockopt, sizeof(sockopt)) < 0)
+        if (setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &sockopt, sizeof(sockopt)) != 0)
         {
             swSysError("setsockopt(TCP_NODELAY) failed.");
         }
@@ -1778,7 +1775,7 @@ static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, i
     //socket recv buffer size
     if (ls->kernel_socket_recv_buffer_size > 0)
     {
-        if (setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &ls->kernel_socket_recv_buffer_size, sizeof(int)))
+        if (setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &ls->kernel_socket_recv_buffer_size, sizeof(int)) != 0)
         {
             swSysError("setsockopt(SO_RCVBUF, %d) failed.", ls->kernel_socket_recv_buffer_size);
         }
@@ -1787,7 +1784,7 @@ static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, i
     //socket send buffer size
     if (ls->kernel_socket_send_buffer_size > 0)
     {
-        if (setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &ls->kernel_socket_send_buffer_size, sizeof(int)) < 0)
+        if (setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &ls->kernel_socket_send_buffer_size, sizeof(int)) != 0)
         {
             swSysError("setsockopt(SO_SNDBUF, %d) failed.", ls->kernel_socket_send_buffer_size);
         }
