@@ -107,7 +107,7 @@ int swPort_listen(swListenPort *ls)
 #ifdef TCP_DEFER_ACCEPT
     if (ls->tcp_defer_accept)
     {
-        if (setsockopt(sock, IPPROTO_TCP, TCP_DEFER_ACCEPT, (const void*) &ls->tcp_defer_accept, sizeof(int)) < 0)
+        if (setsockopt(sock, IPPROTO_TCP, TCP_DEFER_ACCEPT, (const void*) &ls->tcp_defer_accept, sizeof(int)) != 0)
         {
             swSysError("setsockopt(TCP_DEFER_ACCEPT) failed.");
         }
@@ -117,7 +117,7 @@ int swPort_listen(swListenPort *ls)
 #ifdef TCP_FASTOPEN
     if (ls->tcp_fastopen)
     {
-        if (setsockopt(sock, IPPROTO_TCP, TCP_FASTOPEN, (const void*) &ls->tcp_fastopen, sizeof(int)) < 0)
+        if (setsockopt(sock, IPPROTO_TCP, TCP_FASTOPEN, (const void*) &ls->tcp_fastopen, sizeof(int)) != 0)
         {
             swSysError("setsockopt(TCP_FASTOPEN) failed.");
         }
@@ -127,7 +127,7 @@ int swPort_listen(swListenPort *ls)
 #ifdef SO_KEEPALIVE
     if (ls->open_tcp_keepalive == 1)
     {
-        if (setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (void *) &option, sizeof(option)) < 0)
+        if (setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (void *) &option, sizeof(option)) != 0)
         {
             swSysError("setsockopt(SO_KEEPALIVE) failed.");
         }
@@ -365,6 +365,7 @@ static int swPort_onRead_http(swReactor *reactor, swListenPort *port, swEvent *e
     {
         buffer->length += n;
 
+        _parse:
         if (request->method == 0 && swHttpRequest_get_protocol(request) < 0)
         {
             if (request->excepted == 0 && request->buffer->length < SW_HTTP_HEADER_MAX_SIZE)
@@ -372,11 +373,11 @@ static int swPort_onRead_http(swReactor *reactor, swListenPort *port, swEvent *e
                 return SW_OK;
             }
             swoole_error_log(SW_LOG_TRACE, SW_ERROR_HTTP_INVALID_PROTOCOL, "get protocol failed.");
-#ifdef SW_HTTP_BAD_REQUEST_TIP
-            if (swConnection_send(conn, SW_STRL(SW_HTTP_BAD_REQUEST_TIP), 0) < 0)
-            {
-                swSysError("send() failed.");
-            }
+#ifdef SW_USE_HTTP2
+            _bad_request:
+#endif
+#ifdef SW_HTTP_BAD_REQUEST_PACKET
+            swConnection_send(conn, SW_STRL(SW_HTTP_BAD_REQUEST_PACKET), 0);
 #endif
             goto close_fd;
         }
@@ -389,6 +390,11 @@ static int swPort_onRead_http(swReactor *reactor, swListenPort *port, swEvent *e
 #ifdef SW_USE_HTTP2
         else if (request->method == SW_HTTP_PRI)
         {
+            if (unlikely(!port->open_http2_protocol))
+            {
+                goto _bad_request;
+            }
+
             conn->http2_stream = 1;
             swHttp2_send_setting_frame(protocol, conn);
             if (n == sizeof(SW_HTTP2_PRI_STRING) - 1)
@@ -432,7 +438,7 @@ static int swPort_onRead_http(swReactor *reactor, swListenPort *port, swEvent *e
             if (swHttpRequest_get_header_info(request) < 0)
             {
                 /* the request is really no body */
-                if (buffer->length == request->header_length)
+                if (buffer->length >= request->header_length)
                 {
                     /**
                      * send static file content directly in the reactor thread
@@ -442,7 +448,16 @@ static int swPort_onRead_http(swReactor *reactor, swListenPort *port, swEvent *e
                         /**
                          * dynamic request, dispatch to worker
                          */
-                        swReactorThread_dispatch(conn, buffer->str, buffer->length);
+                        swReactorThread_dispatch(conn, buffer->str, request->header_length);
+                        /**
+                         * http pipeline, multi request
+                         */
+                        if (conn->active && buffer->length > request->header_length)
+                        {
+                            swString_pop_front(buffer, request->header_length);
+                            swHttpRequest_clean(request);
+                            goto _parse;
+                        }
                     }
                     swHttpRequest_free(conn);
                     return SW_OK;
