@@ -191,7 +191,7 @@ static DataBuffer task_unpack(swEventData *task_result)
         else
         {
             void *_buffer = retval.alloc((size_t) _pkg.length);
-            if (swoole_sync_readfile(tmp_file_fd, _buffer, _pkg.length) > 0)
+            if (swoole_sync_readfile(tmp_file_fd, _buffer, _pkg.length) != _pkg.length)
             {
                 close(tmp_file_fd);
                 unlink(_pkg.tmpfile);
@@ -350,7 +350,7 @@ bool Server::sendfile(int fd, string &file, off_t offset, size_t length)
     }
     if (file_stat.st_size <= offset)
     {
-        swWarn("file[offset=%ld] is empty.", offset);
+        swWarn("file[offset=%jd] is empty.", (intmax_t) offset);
         return false;
     }
     return serv.sendfile(&serv, fd, (char *) file.c_str(), file.length(), offset, length) == SW_OK;
