@@ -147,17 +147,16 @@ static PHP_METHOD(swoole_channel_coro, __construct)
 {
     zend_long capacity = 1;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "|l", &capacity) == FAILURE)
-    {
-        RETURN_FALSE;
-    }
+    ZEND_PARSE_PARAMETERS_START_EX(ZEND_PARSE_PARAMS_THROW, 0, 1)
+        Z_PARAM_OPTIONAL
+        Z_PARAM_LONG(capacity)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
     if (capacity <= 0)
     {
         capacity = 1;
     }
 
-    php_swoole_check_reactor();
-
     channel_coro *chan_t = swoole_channel_coro_fetch_object(Z_OBJ_P(getThis()));
     chan_t->chan = new Channel(capacity);
     zend_update_property_long(swoole_channel_coro_ce_ptr, getThis(), ZEND_STRL("capacity"), capacity);
@@ -165,8 +164,6 @@ static PHP_METHOD(swoole_channel_coro, __construct)
 
 static PHP_METHOD(swoole_channel_coro, push)
 {
-    PHPCoroutine::check();
-
     Channel *chan = swoole_get_channel(getThis());
     if (chan->is_closed())
     {
@@ -202,8 +199,6 @@ static PHP_METHOD(swoole_channel_coro, push)
 
 static PHP_METHOD(swoole_channel_coro, pop)
 {
-    PHPCoroutine::check();
-
     Channel *chan = swoole_get_channel(getThis());
     if (chan->is_closed())
     {
