@@ -70,7 +70,6 @@ static int php_swoole_event_onRead(swReactor *reactor, swEvent *event)
     zval args[1];
     php_reactor_fd *fd = event->socket->object;
 
-
     args[0] = *fd->socket;
 
     if (sw_call_user_function_ex(EG(function_table), NULL, fd->cb_read, &retval, 1, args, 0, NULL) == FAILURE)
@@ -716,7 +715,8 @@ PHP_FUNCTION(swoole_event_defer)
     defer->callback = &defer->_callback;
     memcpy(defer->callback, callback, sizeof(zval));
     Z_TRY_ADDREF_P(callback);
-    SW_CHECK_RETURN(SwooleG.main_reactor->defer(SwooleG.main_reactor, php_swoole_event_onDefer, defer));
+    SwooleG.main_reactor->defer(SwooleG.main_reactor, php_swoole_event_onDefer, defer);
+    RETURN_TRUE;
 }
 
 PHP_FUNCTION(swoole_event_cycle)
