@@ -287,10 +287,11 @@ PHP_METHOD(swoole_table, __construct)
     zend_long table_size;
     double conflict_proportion = SW_TABLE_CONFLICT_PROPORTION;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|d", &table_size, &conflict_proportion) == FAILURE)
-    {
-        RETURN_FALSE;
-    }
+    ZEND_PARSE_PARAMETERS_START_EX(ZEND_PARSE_PARAMS_THROW, 1, 2)
+        Z_PARAM_LONG(table_size)
+        Z_PARAM_OPTIONAL
+        Z_PARAM_DOUBLE(conflict_proportion)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
     swTable *table = swTable_new(table_size, conflict_proportion);
     if (table == NULL)
