@@ -1,8 +1,8 @@
 <?php
-require_once  __DIR__ . '/config.php'; // (`once` because it may required in skip when we run phpt)
+require_once __DIR__ . '/config.php'; // (`once` because it may required in skip when we run phpt)
 
+error_reporting(E_ALL ^ E_DEPRECATED);
 ini_set('memory_limit', '1024M');
-ini_set('swoole.aio_mode', SWOOLE_AIO_BASE); // SWOOLE_AIO_BASE, SWOOLE_AIO_LINUX
 ini_set("assert.active", 1);
 assert_options(ASSERT_ACTIVE, 1);
 assert_options(ASSERT_WARNING, 1);
@@ -11,12 +11,14 @@ assert_options(ASSERT_QUIET_EVAL, 0);
 
 swoole_async_set([
     'socket_dontwait' => 1,
-    'aio_mode' => SWOOLE_AIO_BASE,
-    'thread_num' => 1,
     'disable_dns_cache' => true,
     'dns_lookup_random' => true,
 ]);
 
+co::set([
+    'socket_timeout' => 5
+]);
+
 if (empty(getenv('SWOOLE_DEBUG'))) {
     if (method_exists('co', 'set')) {
         co::set([
