@@ -37,20 +37,41 @@ define('UDP_SERVER_PORT', 9003);
 define('MYSQL_SERVER_PATH', getenv('MYSQL_SERVER_PATH') ?:
     (IS_IN_TRAVIS ? TRAVIS_DIR_PATH . '/data/run/mysqld/mysqld.sock' :
         (IS_MAC_OS ? '/tmp/mysql.sock' : '/var/run/mysqld/mysqld.sock')));
-define('MYSQL_SERVER_HOST', IS_IN_TRAVIS ? 'mysql' : '127.0.0.1');
-define('MYSQL_SERVER_PORT', 3306);
-define('MYSQL_SERVER_USER', 'root');
-define('MYSQL_SERVER_PWD', 'root');
-define('MYSQL_SERVER_DB', 'test');
+define('MYSQL_SERVER_HOST', getenv('MYSQL_SERVER_HOST') ?: (IS_IN_TRAVIS ? 'mysql' : '127.0.0.1'));
+define('MYSQL_SERVER_PORT', (int)(getenv('MYSQL_SERVER_PORT') ?: 3306));
+define('MYSQL_SERVER_USER', getenv('MYSQL_SERVER_USER') ?: 'root');
+define('MYSQL_SERVER_PWD', getenv('MYSQL_SERVER_PWD') ?: 'root');
+define('MYSQL_SERVER_DB', getenv('MYSQL_SERVER_DB') ?: 'test');
 
 /** ============== Redis ============== */
 define('REDIS_SERVER_PATH', getenv('REDIS_SERVER_PATH') ?:
     (IS_IN_TRAVIS ? TRAVIS_DIR_PATH . '/data/run/redis/redis.sock' :
         (IS_MAC_OS ? '/tmp/redis.sock' : '/var/run/redis/redis-server.sock')));
-define('REDIS_SERVER_HOST', IS_IN_TRAVIS ? 'redis' : '127.0.0.1');
-define('REDIS_SERVER_PORT', 6379);
-define('REDIS_SERVER_PWD', 'root');
-define('REDIS_SERVER_DB', 0);
+define('REDIS_SERVER_HOST', getenv('REDIS_SERVER_HOST') ?: (IS_IN_TRAVIS ? 'redis' : '127.0.0.1'));
+define('REDIS_SERVER_PORT', (int)(getenv('REDIS_SERVER_PORT') ?: 6379));
+define('REDIS_SERVER_PWD', getenv('REDIS_SERVER_PWD') ?: 'root');
+define('REDIS_SERVER_DB', (int)(getenv('REDIS_SERVER_DB') ?: 0));
+
+/** ============== HttpBin ============== */
+if (IS_IN_TRAVIS) {
+    define('HTTPBIN_SERVER_HOST', 'httpbin');
+    define('HTTPBIN_SERVER_PORT', 80);
+    define('HTTPBIN_LOCALLY', true);
+} elseif (!empty($info = `docker ps 2>&1 | grep httpbin 2>&1`) &&
+    preg_match('/\s+?[^:]+:(\d+)->\d+\/tcp\s+/', $info, $matches) &&
+    is_numeric($matches[1])
+) {
+    define('HTTPBIN_SERVER_HOST', '127.0.0.1');
+    define('HTTPBIN_SERVER_PORT', (int)$matches[1]);
+    define('HTTPBIN_LOCALLY', true);
+} elseif (getenv('HTTPBIN_SERVER_HOST')) {
+    define('HTTPBIN_SERVER_HOST', getenv('HTTPBIN_SERVER_HOST'));
+    define('HTTPBIN_SERVER_PORT', (int)getenv('HTTPBIN_SERVER_PORT'));
+    define('HTTPBIN_LOCALLY', true);
+} else {
+    define('HTTPBIN_SERVER_HOST', 'httpbin.org');
+    define('HTTPBIN_SERVER_PORT', 80);
+}
 
 /** =============== IP ================ */
 define('IP_REGEX', '/^(?:[\d]{1,3}\.){3}[\d]{1,3}$/');
@@ -65,7 +86,7 @@ define('SOCKS5_PROXY_PORT', IS_MAC_OS ? 1086 : 1080);
 define('PRESSURE_LOW', 1);
 define('PRESSURE_MID', 2);
 define('PRESSURE_NORMAL', 3);
-define('PRESSURE_LEVEL', USE_VALGRIND ? PRESSURE_LOW : IS_IN_TRAVIS ? PRESSURE_MID : PRESSURE_NORMAL);
+define('PRESSURE_LEVEL', USE_VALGRIND ? PRESSURE_LOW : (IS_IN_TRAVIS || swoole_cpu_num() === 1) ? PRESSURE_MID : PRESSURE_NORMAL);
 
 /** ============== Time ============== */
 define('SERVER_PREHEATING_TIME', 0.1);
